from rest_framework_json_api import renderers, utils
from rest_framework_json_api.serializers import PolymorphicModelSerializer

from .mixins import SerializationType


def _get_resource_name(context, expand_polymorphic_types=False):
    """
    Return the name of a resource with a special override for ConfigurableGenericViewSet's multiple serializers
    """
    if CGVSJsonRenderer.original_get_resource_name is None:
        raise Exception('original_get_resource_name was not captured')

    view = context.get('view')
    is_response = context.get('response', False)

    try:
        original_view_resource_name = getattr(view, 'resource_name', None)

        # If view.resource_name was set, respect it
        if original_view_resource_name:
            raise AttributeError

        # If view is not a ConfigurableGenericViewSet, don't try to get configurable serializer
        from .viewsets import ConfigurableGenericViewSet
        if not isinstance(view, ConfigurableGenericViewSet):
            raise AttributeError

        serializer = view.get_serializer_class(
            serialization_type=SerializationType.RESPONSE if is_response else SerializationType.REQUEST)

        if expand_polymorphic_types and issubclass(serializer, PolymorphicModelSerializer):
            resource_name = serializer.get_polymorphic_types()
        else:
            resource_name = utils.get_resource_type_from_serializer(serializer)

        # Force the default method to find our serializer at view.resource_name and reset when we're done
        setattr(view, 'resource_name', resource_name)
        # pylint: disable=not-callable
        resource_name = CGVSJsonRenderer.original_get_resource_name(context, expand_polymorphic_types)
        setattr(view, 'resource_name', original_view_resource_name)

        return resource_name

    # For any exceptions above, fallback to original method
    except AttributeError:
        # pylint: disable=not-callable
        return CGVSJsonRenderer.original_get_resource_name(context, expand_polymorphic_types)


class CGVSJsonRenderer(renderers.JSONRenderer):
    """JSONRenderer that is aware of the multiple serializers in ConfigurableGenericViewSet
    This also affects the JSONParser since we are patching method in the util package
    """
    original_get_resource_name = None

    if original_get_resource_name is None:
        original_get_resource_name = utils.get_resource_name
        utils.get_resource_name = _get_resource_name



# flake8: noqa
"""
Copyright 2019 ShipChain, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from .json_asserter import \
    json_asserter, \
    AssertionHelper, \
    JsonAsserterMixin

from .helpers import\
    create_form_content, \
    datetimeAlmostEqual, \
    get_jwt, \
    generate_vnd_json, \
    random_location,\
    random_timestamp, \
    replace_variables_in_string, \
    GeoCoderResponse

from .mocked_rpc_responses import \
    mocked_rpc_response, \
    mocked_wallet_error_creation,\
    mocked_wallet_invalid_creation, \
    mocked_wallet_valid_creation, \
    second_mocked_wallet_valid_creation,\
    invalid_eth_amount, \
    invalid_ship_amount, \
    valid_eth_amount

from ..utils import validate_uuid4  # Imported for backwards compatibility usage of `from shipchain_common.test_utils`



"""
Copyright 2019 ShipChain, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import pytz
import re
import random
from datetime import datetime, timedelta
from uuid import uuid4

import jwt
from django.conf import settings
from django.test.client import encode_multipart
from rest_framework_simplejwt.utils import aware_utcnow, datetime_to_epoch


def create_form_content(data):
    boundary_string = 'BoUnDaRyStRiNg'

    content = encode_multipart(boundary_string, data)
    content_type = 'multipart/form-data; boundary=' + boundary_string

    return content, content_type


def datetimeAlmostEqual(dt1, dt2=None, ms_threshold=None):
    ms_threshold = settings.MILLISECONDS_THRESHOLD if ms_threshold is None else ms_threshold

    if not dt2:
        dt2 = datetime.now().replace(tzinfo=pytz.UTC)

    return dt1 - timedelta(milliseconds=ms_threshold) <= dt2 <= dt1 + timedelta(milliseconds=ms_threshold)


def get_jwt(exp=None, iat=None, sub='00000000-0000-0000-0000-000000000000', username='fake@shipchain.io', **kwargs):
    payload = {'email': username, 'username': username, 'sub': sub,
               'aud': '892633', 'jti': uuid4().hex}

    for prop_name, prop_value in kwargs.items():
        if prop_value is not None:
            # we test 'prop_value' against None to avoid being caught
            # with a boolean 'prop_value = False' which will be false
            # but needs to be set
            payload[prop_name] = prop_value

    now = aware_utcnow()
    if exp:
        payload['exp'] = exp
    else:
        payload['exp'] = datetime_to_epoch(now + timedelta(minutes=5))

    if iat:
        payload['iat'] = iat
    else:
        payload['iat'] = datetime_to_epoch(now)

    return jwt.encode(payload=payload, key=settings.SIMPLE_JWT['PRIVATE_KEY'], algorithm='RS256',
                      headers={'kid': '230498151c214b788dd97f22b85410a5'}).decode('utf-8')


def generate_vnd_json(attributes, object_type, object_id=None):
    """
    Generate Vendor API Json format.  Optionally include `object_id` for PUT/PATCH operations
    """
    data = {
        'data': {
            'type': object_type,
            'attributes': attributes
        }
    }

    if object_id is not None:
        data['data']['id'] = object_id

    return data


def random_location():
    """
    :return: Randomly generated location geo point.
    """
    return {
        "geometry": {
            "coordinates": [random.uniform(-180, 180), random.uniform(-90, 90)],
            "type": "Point"
        },
        "properties": {
            "source": "Gps",
            "time": random_timestamp(),
            "uncertainty": random.randint(0, 99)
        },
        "type": "Feature"
    }


def random_timestamp():
    """
    Returns a randomly generated datetime timestamp between year 2016 and 2019
    Example: 2017-09-16T19:42:21.7396
    """
    def is_one(a, b):
        value = str(random.randint(a, b))
        if len(value) == 1:
            return '0' + value
        return value

    return f'201{random.randint(6,9)}-{is_one(1,9)}-{is_one(1,30)}T{is_one(0,24)}:{is_one(0,59)}:{is_one(0,59)}.' \
        f'{random.randint(1000,9999)}'


def replace_variables_in_string(string, parameters):
    matches = re.findall(r"<<(\w+?)>>", string)
    for match in matches:
        string = string.replace(f"<<{match}>>", parameters[match])
    return string


class GeoCoderResponse:
    def __init__(self, status, point=None):
        self.ok = status
        self.xy = point



"""
Copyright 2019 ShipChain, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from rest_framework.viewsets import GenericViewSet
from rest_framework.settings import api_settings

from . import mixins

FORMAT_SUFFIX = api_settings.FORMAT_SUFFIX_KWARG


class ActionConfiguration:
    """
    Class to hold the configuration settings for a specific Action within a ViewSet.
    All values are optional and default to None if not provided.  Validations run against the
    default actions (CRUDL) to ensure no invalid or conflicting settings are provided.
    """

    class ResponseSerializers:
        """
        Response can be serialized multiple ways depending on FORMAT_SUFFIX_KWARG.
        This class holds references to the serializers used for each case.
        """
        def __init__(self, **kwargs):
            self.default = None
            self.__dict__.update(**kwargs)
            if self.default is None:
                raise AttributeError(f'response_serializer needs a default provided')

    # pylint: disable=too-many-arguments
    def __init__(self,
                 serializer=None,
                 request_serializer=None,
                 response_serializer=None,
                 request_validation=None,
                 permission_classes=None,
                 required_user_permissions=None,
                 success_status=None):

        self.__action_validation_completed = False
        self.__serializer_standardization_completed = False

        self.serializer = serializer
        self.request_serializer = request_serializer
        self.response_serializer = response_serializer
        self.request_validation = request_validation
        self.permission_classes = permission_classes
        self.required_user_permissions = required_user_permissions
        self.success_status = success_status

        self.re_serialize_response = False

        self._validate_configuration()

    def _validate_configuration(self):
        """
        Validate that the combination of provided configuration options are valid.

        `serializer` can be provided as a the serializer class for both the incoming and outgoing data.
        Or the specific `request_serializer` and `response_serializer` can be set.  But if `serializer`
        is set, there should not be a setting for `request_serializer` or `response_serializer`.

        Ensures request_validation is a proper boolean.

        :raises AttributeError:
        """

        if self.serializer and (self.request_serializer or self.response_serializer):
            raise AttributeError('serializer cannot be provided if either '
                                 'request_serializer or response_serializer are provided')

        if self.request_validation is not None and not isinstance(self.request_validation, bool):
            raise AttributeError('request_validation should be a boolean value')

        if self.required_user_permissions is not None and isinstance(self.required_user_permissions, str):
            self.required_user_permissions = [self.required_user_permissions]

    def validate_action(self, action):
        """
        Validate that the provided configuration options are valid for the specific Action
        :raises AttributeError:
        """
        if self.__action_validation_completed:
            return

        if action in ['retrieve', 'list', 'destroy']:
            for attr in ['request_serializer', 'request_validation', 'success_status']:
                if getattr(self, attr, None):
                    raise AttributeError(f'{attr} not valid for action {action}')

        if action in ['retrieve', 'list']:
            if self.serializer and self.response_serializer:
                raise AttributeError(f'Only one of serializer or response_serializer are valid for action {action}')

        if action in ['destroy']:
            if self.serializer or self.request_serializer or self.response_serializer:
                raise AttributeError(f'serializers are not valid for action {action}')

        self.__action_validation_completed = True

    def standardize_serializer_properties(self):
        """
        Final serializer options should be in `request_serializer` and `response_serializer` and not in `serializer`.
        If there was an explicit `response_serializer` set, then it is likely different than the request serializer
        (whether set explicitly in configuration or via the fallback default `serializer_class`).  In this case we
        will re-serialize the data in the response using the response_serializer.

        Convert the response_serializer value in to a ResponseSerializers object.  If the provided value is not a
        dictionary, then set the `default` property to the serializer class provided.  If a dictionary was provided,
        ensure the `default` key exists and create the ResponseSerializers object with the provided property/values.

        :raises AttributeError:
        """
        if self.__serializer_standardization_completed:
            return

        if self.serializer:
            self.request_serializer = self.serializer
            self.response_serializer = self.serializer
            self.serializer = None

        elif self.request_serializer or self.response_serializer:
            self.re_serialize_response = True

        if self.response_serializer:
            if not isinstance(self.response_serializer, ActionConfiguration.ResponseSerializers):
                if isinstance(self.response_serializer, dict):
                    self.response_serializer = ActionConfiguration.ResponseSerializers(**self.response_serializer)
                else:
                    self.response_serializer = ActionConfiguration.ResponseSerializers(default=self.response_serializer)

        self.__serializer_standardization_completed = True

    @property
    def raise_validation(self):
        return self.request_validation if self.request_validation is not None else True


class ConfigurableGenericViewSet(GenericViewSet):
    """
    The ConfigurableGenericViewSet class does not provide any actions by default,
    but does include the base set of generic view behavior, as well as the
    logic to handle the configuration via processing ActionConfiguration objects.
    """
    configuration = None
    action_user_permissions = None
    default_required_user_permissions = None

    def __init__(self, **kwargs):
        super(ConfigurableGenericViewSet, self).__init__(**kwargs)

        if issubclass(self.__class__, mixins.MultiSerializerViewSetMixin):
            raise AttributeError(f'Cannot use MultiSerializerViewSetMixin when using ConfigurableGenericViewSet')

        if issubclass(self.__class__, mixins.MultiPermissionViewSetMixin):
            raise AttributeError(f'Cannot use MultiPermissionViewSetMixin when using ConfigurableGenericViewSet')

    def _process_configurations(self):

        if self.configuration is None:
            raise AttributeError(f'{self.__class__.__name__} should include a `configuration` attribute.')

        for action, config in self.configuration.items():
            config.validate_action(action)
            config.standardize_serializer_properties()

            if config.required_user_permissions:
                if self.action_user_permissions is None:
                    self.action_user_permissions = {}
                self.action_user_permissions[action] = config.required_user_permissions

        if self.default_required_user_permissions:
            if self.action_user_permissions is None:
                self.action_user_permissions = {}
            if self.action not in self.action_user_permissions:
                self.action_user_permissions[self.action] = self.default_required_user_permissions

    def get_configuration(self):
        """
        Return the ActionConfiguration for the specific action taking place.  If no defined ActionConfiguration
        exists then return an empty ActionConfiguration.  All ActionConfiguration properties are initialized to None
        """
        self._process_configurations()

        action = self.action

        # PUT and PATCH should use the same configuration
        # pylint: disable=unsupported-membership-test
        if action == 'partial_update' and 'partial_update' not in self.configuration:
            action = 'update'

        # pylint: disable=unsupported-membership-test
        if action in self.configuration:
            return self.configuration[action]  # pylint: disable=unsubscriptable-object

        return ActionConfiguration()

    def get_permissions(self):
        """
        For the given ActionConfiguration, return the defined permission_classes.  If no permission_classes exist
        in the ActionConfiguration, then return the permission_classes as defined on the viewset (which will
        still fall back to the global default setting).
        """
        config = self.get_configuration()
        permission_classes = self.permission_classes

        if config.permission_classes:
            permission_classes = config.permission_classes

        return [permission() for permission in permission_classes]

    def get_serializer(self, *args, **kwargs):
        """
        Return the serializer instance that should be used for validating and
        deserializing input, and for serializing output.

        This accepts a kwarg `serialization_type` that defaults to REQUEST if not provided.  The associated
        configurable mixins for the default actions provide a serialization_type of RESPONSE in the actions
        where the response may be serialized with a separate class.
        """
        serialization_type = kwargs.pop('serialization_type', mixins.SerializationType.REQUEST)
        serializer_class = self.get_serializer_class(serialization_type)
        kwargs['context'] = self.get_serializer_context()
        return serializer_class(*args, **kwargs)

    # pylint: disable=arguments-differ
    def get_serializer_class(self, serialization_type=mixins.SerializationType.REQUEST):
        """
        Return the serializer class for the given action and type based on the ActionConfiguration and the
        provided `serialization_type` parameter.  Additionally, if the `format` kwarg is present, then use
        that as an additional lookup field in the ResponseSerializers object.  Failing all this, use the
        super get_serializer_class which should return the value as defined on the ViewSet.
        """
        config = self.get_configuration()

        if serialization_type == mixins.SerializationType.REQUEST and config.request_serializer:
            return config.request_serializer

        if serialization_type == mixins.SerializationType.RESPONSE and config.response_serializer:
            if FORMAT_SUFFIX in self.kwargs and hasattr(config.response_serializer, self.kwargs[FORMAT_SUFFIX]):
                return getattr(config.response_serializer, self.kwargs[FORMAT_SUFFIX])
            return config.response_serializer.default

        return super(ConfigurableGenericViewSet, self).get_serializer_class()


class ConfigurableModelViewSet(mixins.ConfigurableCreateModelMixin,
                               mixins.ConfigurableRetrieveModelMixin,
                               mixins.ConfigurableUpdateModelMixin,
                               mixins.ConfigurableDestroyModelMixin,
                               mixins.ConfigurableListModelMixin,
                               ConfigurableGenericViewSet):
    """
    A viewset that provides *configurable* `create()`, `retrieve()`, `update()`,
    `partial_update()`, `destroy()` and `list()` actions.
    """



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['shipchain_common',
 'shipchain_common.custom_logging',
 'shipchain_common.test_utils']

package_data = \
{'': ['*']}

install_requires = \
['aws-requests-auth>=0.4,<0.5',
 'boto3>=1.9,<1.10',
 'cryptography>=2.5,<2.6',
 'django-enumfields>=0.10.0,<0.11.0',
 'django-influxdb-tagged-metrics==1.3.3',
 'django>=2.2.7,<2.3.0',
 'djangorestframework-jsonapi>=3,<4',
 'djangorestframework>=3.10,<3.11',
 'drf-nested-routers>=0.91.0,<0.92.0',
 'python-dateutil>=2.7.5,<2.8.0',
 'requests>=2.21,<3.0']

entry_points = \
{'pytest11': ['json_asserter = shipchain_common.test_utils.json_asserter']}

setup_kwargs = {
    'name': 'shipchain-common',
    'version': '1.0.16',
    'description': "A PyPI package containing shared code for ShipChain's Python/Django projects.",
    'long_description': '# Shipchain Common Python Library\n\nA PyPI package containing shared code for ShipChain\'s Python/Django projects\n\n\n## Pytest Fixtures\n\nWhen shipchain-common is installed, a pytest plugin named `json_asserter` is automatically registered.  This plugin is\n designed for writing concise pytest cases that make json_asserter about responses from a Django Rest Framework API. Most \n of the functionality is tailored to the `application/vnd.api+json` response type, but should still be usable for\n plain `application/json` responses.\n \n### json_asserter Fixture\n\nThe `json_asserter` fixture exposes several methods for testing specific HTTP Status codes as well as a class for\n building consistent entity references that must be found within the responses.\n \n#### Usage with application/vnd.api+json\n\nThis is the default when utilizing the `json_asserter`.  If the response does not conform to the \n[JSON Api standard](https://jsonapi.org/), the assertions will fail.\n \n##### Asserting Error Responses\n\nTo assert that a given response must have an error status, there are several 400-level response methods.  With the\n exception of the HTTP_400 method, each of these include the default error message for ease of use.\n \nThe following will assert that the response status was 403 and that the default error message ("You do not have\n permission to perform this action") is present.\n \n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_403(response)\n```\n \nIf a different error message should exist, or when checking the error of a 400 response, the specific error may\n be provided as an argument.\n \n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_400(response, error=\'Specific error message that should be in the respose\')\n```\n\n##### Asserting Successful Responses\n\nTo assert that a given response must have status 200, call the HTTP_200 method with only the Response object:\n\n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_200(response)\n```\n \nWhile this is valid, it is **very strongly** recommended to include additional details about the data present in the\n response. There are two ways to provide the data; however only one way can be used at a time in a given invocation.\n \n###### Simple Usage\n \nFor simple responses, the easiest way to specify required data in the responses is by directly specifying the\n Resource Type `resource`, the Resource Identifier `pk`, as well as any specific Attributes of the resource\n  `attributes`. \n  \n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_200(response, \n                    resource=\'User\', \n                    pk=\'4b56399d-3155-4fe5-ba4a-9718289a78b7\', \n                    attributes={\'username\': \'example_user\'})\n```\n\nThis will throw an assertion if the response is not for the resource type `User` with id \n`4b56399d-3155-4fe5-ba4a-9718289a78b7` and with _at least_ the attribute username `example_user`.  If the response\n includes _additional_ attributes that are not listed in the call to the json_asserter method, they are ignored.  The\n methods check partial objects and do not require that every attribute in the response must be defined in the\n assertion.\n   \nIt is also possible to assert only on the resource type and id without providing attributes.  This is useful if you\n are testing a response that generates content for the fields that may not be known prior to obtaining the response. \n Additionally, providing only the attributes and not the type and id will check only that an object in the response\n has those attributes, regardless of resource type or id.\n\n###### Advanced Usage\n \nFor responses where the associated Relationship and any extra Included resources are important, those can be included\n in the assertion.\n  \n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_200(response,\n                    entity_refs=json_asserter.EntityRef(\n                        resource=\'User\', \n                        pk=\'4b56399d-3155-4fe5-ba4a-9718289a78b7\', \n                        attributes={\'username\': \'example_user\'},\n                        relationships={\n                            \'manager\': json_asserter.EntityRef( \n                                resource=\'User\', \n                                pk=\'88e38305-9775-4b34-95d0-4e935bb7156c\')}),\n                    included=json_asserter.EntityRef(\n                        resource=\'User\', \n                        pk=\'88e38305-9775-4b34-95d0-4e935bb7156c\', \n                        attributes={\'username\': \'manager_user\'}))\n```\n\nThis requires the same original record in the response, but now also requires that there be _at least_ one relationship\n named `manager` with the associated User and that User must be present (with at least the one attribute) in the\n `included` property of the response.\n \nThe above example utilizes the `EntityRef` exposed via the `json_asserter` fixture.  This is a reference to a single\n entity defined by a combination of: ResourceType, ResourceID, Attributes, and Relationships. When providing the\n `entity_refs` argument to an assertion, you cannot provide any of the following arguments to the assertion directly:\n `resource`, `pk`, `attributes`, or `relationships`.\n \nWhen providing `included` json_asserter, you can provide either a single EntityRef or a list of EntityRef instances.  If\n a list is provided, _all_ referenced entities must be present in the `included` property of the response. As they do\n for the simple usage above, The same assertion rules apply here regarding providing a combination of `resource`, \n `pk`, and `attributes`.\n \nThe `entity_refs` parameter can be a list of EntityRef instances as well. However, this is only valid for List\n responses.  If a list of entity_refs is provided for a non-list response, an assertion will occur.  To assert that a\n response is a list, the parameter `is_list=True` must be provided. You can provide either a single EntityRef or a\n list of EntityRef instances.  If a list is provided, _all_ referenced entities must be present in the list of\n returned data.\n\n#### Usage with application/json\n\nSupport is included for making assertions on plain JSON responses with `json_asserter`. To ignore the JSON API specific \n assertions, you must provide the `vnd=False` parameter.  Only the `attributes` parameter is valid as there are no\n relationships or included properties in a plain json response.\n \nGiven this response:\n\n```json\n{\n    "id": "07b374c3-ed9b-4811-901a-d0c5d746f16a",\n    "name": "example 1",\n    "field_1": 1,\n    "owner": {\n        "username": "user1"\n    }\n}\n```\n\nAsserting the top level attributes as well as nested attributes is possible using the following call:\n\n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_200(response, \n                    vnd=False,\n                    attributes={\n                        \'id\': \'07b374c3-ed9b-4811-901a-d0c5d746f16a\',\n                        \'owner\': {\n                            \'username\': \'user1\'\n                        }\n                    })\n```\n\nFor a list response:\n\n```json\n[{\n    "username": "user1",\n    "is_active": False\n},\n{\n    "username": "user2",\n    "is_active": False\n},\n{\n    "username": "user3",\n    "is_active": False\n}]\n```\n\nIt is possible to assert that one or many sets of attributes exist in the response:\n```python\nresponse = api_client.get(self.detail_url)\njson_asserter.HTTP_200(response, \n                    vnd=False,\n                    is_list=True,\n                    attributes=[{\n                        "username": "user1",\n                        "is_active": False\n                    }, {\n                        "username": "user3",\n                        "is_active": False\n                    }])\n```\n\n#### Mixin Usage\n\nIf there is a class where every test may wish to use the `json_asserter`, than it may be easier to use to the `JsonAsserterMixin` found in `shipchain_common.test_utils`.\nThis will automatically add the `json_asserter` and set it as a class attribute before the tests are run.\nThis allows you to just call `self.json_asserter`, allowing for cleaner unit tests imports. \n',
    'author': 'Adam Hodges',
    'author_email': 'ahodges@shipchain.io',
    'url': 'https://github.com/ShipChain/python-common',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<3.8',
}


setup(**setup_kwargs)



"""
Copyright 2019 ShipChain, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from unittest.mock import Mock
from requests.models import Response


def mocked_rpc_response(json, content=None, code=200):
    response = Mock(spec=Response)
    response.status_code = code
    response.content = content
    response.json.return_value = json
    return response


def invalid_eth_amount():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {
            "success": True,
            "ether": "1000000",
            "ship": "1000000000000000000"
        },
        "id": 0
    })


def invalid_ship_amount():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {
            "success": True,
            "ether": "1000000000000000000",
            "ship": "1000000"
        },
        "id": 0
    })


def mocked_wallet_valid_creation():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {
            "success": True,
            "wallet": {
                "id": "d5563423-f040-4e0d-8d87-5e941c748d91",
                "public_key": "a07d45389b1a3b40c6784f749a1d616b4c6f0dba195c0348aef81e9b4c8a7f5c13c0d9b8c360cd2307683a2"
                              "d8f20bd7d801c7a11f4681440f11372f2de465942",
                "address": "0x94Fad76b5Be2b746598BCe12e7b45D7C06D8DA1F"
            }
        },
        "id": 0
    })


def mocked_wallet_invalid_creation():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {
            "success": True,
            "wallet": {
                "public_key": "a07d45389b1a3b40c6784f749a1d616b4c6f0dba195c0348aef81e9b4c8a7f5c13c0d9b8c360cd2307683a2"
                              "d8f20bd7d801c7a11f4681440f11372f2de465942",
                "address": "0x94Fad76b5Be2b746598BCe12e7b45D7C06D8DA1F"
            }
        },
        "id": 0
    })


def mocked_wallet_error_creation():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {},
        "id": 0
    })


def second_mocked_wallet_valid_creation():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {
            "success": True,
            "wallet": {
                "id": "256e621b-2d42-4bf2-ac76-27336d4bf770",
                "public_key": "234111c81a928562e114b9b137dac3c36d7fac5fb6551042608a69c4838335646a641059995510793d9be5f"
                              "c9ea4dd0c5180aafff831462319b3c6878812f987",
                "address": "0x3fB9Ff55672084f3A34E4C77dACF5f3a8D71037a"
            }
        },
        "id": 0
    })


def valid_eth_amount():
    return mocked_rpc_response({
        "jsonrpc": "2.0",
        "result": {
            "success": True,
            "ether": "1000000000000000000",
            "ship": "1000000000000000000"
        },
        "id": 0
    })


"""
Copyright 2019 ShipChain, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from django.conf import settings
from django.core.cache import cache
from rest_framework.exceptions import AuthenticationFailed
from rest_framework.permissions import BasePermission
from rest_framework_simplejwt.authentication import JWTTokenUserAuthentication
from rest_framework_simplejwt.models import TokenUser

from .utils import parse_dn


PASSIVE_JWT_AUTHENTICATION = JWTTokenUserAuthentication()


def passive_credentials_auth(jwt):
    if jwt == "":
        raise AuthenticationFailed('No JWT provided with request')
    return PASSIVE_JWT_AUTHENTICATION.get_user(PASSIVE_JWT_AUTHENTICATION.get_validated_token(jwt))


def get_jwt_from_request(request):
    """
    This is for retrieving the decoded JWT from the a request via the simplejwt authenticator.
    """
    if not settings.PROFILES_ENABLED:
        return None
    return (request.authenticators[-1].get_raw_token(request.authenticators[-1].get_header(request)).decode()
            if request.authenticators else None)


def is_internal_call(request):
    return ('X_NGINX_SOURCE' in request.META and request.META['X_NGINX_SOURCE'] == 'internal'
            and request.META['X_SSL_CLIENT_VERIFY'] == 'SUCCESS')


class InternalRequest(BasePermission):
    def has_permission(self, request, view):
        if settings.ENVIRONMENT in ('LOCAL', 'INT'):
            return True
        if is_internal_call(request):
            certificate_cn = parse_dn(request.META['X_SSL_CLIENT_DN'])['CN']
            return certificate_cn == f'{self.SERVICE_NAME}.{settings.ENVIRONMENT.lower()}-internal'
        return False


class EngineRequest(InternalRequest):
    SERVICE_NAME = 'engine'


class PermissionedTokenUser(TokenUser):
    """
    This Requires the JWT from Profiles to have been generated with the `permissions` scope
    Override the default class by setting SIMPLE_JWT['TOKEN_USER_CLASS'] = 'path.to.this.class'
    """

    def save(self):
        raise NotImplementedError('Token users have no DB representation')

    def delete(self):
        raise NotImplementedError('Token users have no DB representation')

    def set_password(self, raw_password):
        raise NotImplementedError('Token users have no DB representation')

    def check_password(self, raw_password):
        raise NotImplementedError('Token users have no DB representation')

    def _get_permission_cache_key(self):
        """
        Build a unique cache key for this specific JWT.
        If no `jti`, `at_hash`, or `sub` and `exp`, then return None
        """
        unique_key = self.token.get('jti')

        if not unique_key:
            unique_key = self.token.get('at_hash')

        if not unique_key:
            sub = self.token.get("sub")
            exp = self.token.get("exp")
            if sub and exp:
                unique_key = f'{sub}.{exp}'

        return unique_key

    def _get_permission_cache_life(self):
        """
        Determine cache life from JWT.  If exp or iat are not present, or
        if calculation results in an invalid life, return the fallback_life
        """
        fallback_life = 300

        exp = self.token.get("exp")
        iat = self.token.get("iat")

        if not exp or not iat:
            return fallback_life

        life = exp - iat

        if not life or life <= 0:
            return fallback_life

        return life

    def get_all_permissions(self, obj=None):
        """
        For each Feature/FeaturePermission, build a dot-pathed permission
        These values are cached if we can find a suitable unique key in the token.
        This prevents re-parsing the permissions over the lifetime of this token
        as they will not change until a new token is received
        """
        permissions = None
        unique_key = self._get_permission_cache_key()

        if unique_key:
            permissions = cache.get(unique_key)

        if not permissions:
            features = self.token.get('features')
            if not features:
                return []

            permissions = []
            for feature in features:
                for permission in features[feature]:
                    permissions.append(f'{feature}.{permission}')

            if unique_key:
                cache.set(unique_key, permissions, self._get_permission_cache_life())

        return permissions

    def has_perm(self, perm, obj=None):
        """
        Validate perm is in token feature permissions
        """
        return perm in self.get_all_permissions(obj)

    def has_perms(self, perm_list, obj=None):
        """
        Validate perm_list is in token feature permissions
        """
        return all(self.has_perm(perm, obj) for perm in perm_list)
